# ========== aiちゅーんスクリプトをテキストファイル化するツール ==========
#
#  2010-02-07 版
#  https://hashimo.ddo.jp/
#

  use strict;
  use utf8;
  use Encode;
  use Config::Tiny;
  use IO::Handle;
  use Cwd;
  use FindBin::Real;

  my ($indir, $outdir, $indent, $strb, $stre, $unkb, $unke, $dh, $cfgpath);
  my ($ecd, $bom);

  binmode STDOUT, ':crlf:encoding(cp932)';
  binmode STDERR, ':crlf:encoding(cp932)';
  binmode STDIN, ':crlf:encoding(cp932)';

  #
  # 設定ファイルのパス名を決定
  {
    my ($f, $d, $r, $i);
    $f = FindBin::Real::Script();
    $r = FindBin::Real::Bin();
    $d = getcwd();
    $i = rindex($f, '.');
    $f = substr($f, 0, $i) if ($i >= 0);
    $f .= '.cfg';
    $r .= '/' unless (substr($r, -1, 1) eq '/');
    $d .= '/' unless (substr($d, -1, 1) eq '/');
    $r .= $f;
    $d .= $f;
    if (-e $d) {
      $cfgpath = $d;  # カレントフォルダに基づく
    }
    elsif (-e $r) {
      $cfgpath = $r;  # 本スクリプトの位置に基づく
    }
    else {
      print STDERR decode('cp932', $f) . ' が存在しません';
      exit 1;
    }
  }

  #
  # 設定ファイルを読み出す
  {
    my ($cfg);
    $cfg = Config::Tiny -> new();
    if ($cfg = Config::Tiny -> read($cfgpath)) {
      $indir = $cfg -> {'Path'} -> {'InputDirectory'};
      if ($indir eq '') {
        print STDERR 'InputDirectoryが設定されていません';
        exit 1;
      }
      $outdir = $cfg -> {'Path'} -> {'OutputDirectory'};
      $outdir = '.\\' if ($outdir eq '');
      if ($cfg -> {'OutputFormat'} -> {'Bom'}) {
        $bom = "\x{feff}";
      }
      else {
        $bom = '';
      }
      $ecd = $cfg -> {'OutputFormat'} -> {'Encoding'};
      if ($ecd) {
        $ecd = lc($ecd);
        if ($ecd =~ /^utf.*8$/) {
          $ecd = 'utf8';
        }
        elsif ($ecd =~ /^s.*jis$/ or $ecd =~ /^cp932$/) {
          $ecd = 'encoding(cp932)';
        }
        elsif ($ecd =~ /^euc/) {
          $ecd = 'encoding(euc-jp)';
        }
        else {
          print STDERR 'Encodingオプションの指定値が不正です';
          exit 1;
        }
      }
      else {
        $ecd = 'utf8';
      }
      $indent = decode('cp932', $cfg -> {'OutputFormat'} -> {'Indent'});
      $strb = decode('cp932', $cfg -> {'OutputFormat'} -> {'StringBegin'});
      $stre = decode('cp932', $cfg -> {'OutputFormat'} -> {'StringEnd'});
      $unkb = decode('cp932', $cfg -> {'OutputFormat'} -> {'UnknownBegin'});
      $unke = decode('cp932', $cfg -> {'OutputFormat'} -> {'UnknownEnd'});
      $indir .= '\\' if (substr($indir, -1, 1) ne '\\');
      $outdir .= '\\' if (substr($outdir, -1, 1) ne '\\');
      $indent = $1 if ($indent =~ /^"(.*)"$/);
      $strb = $1 if ($strb =~ /^"(.*)"$/);
      $stre = $1 if ($stre =~ /^"(.*)"$/);
      $unkb = $1 if ($unkb =~ /^"(.*)"$/);
      $unke = $1 if ($unke =~ /^"(.*)"$/);
    }
    else {
      print STDERR decode('cp932', $cfgpath);
      print STDERR ' にアクセスできません';
      exit 1;
    }
  }

  #
  # スクリプトファイルに対して順に処理する

  $dh = IO::Handle -> new();
  if (opendir $dh, $indir) {
    my ($infile);
    foreach $infile (readdir $dh) {
      my ($pa);
      $pa = $indir . $infile;
      next if (-d $pa);
      next unless (lc(substr($infile, -4)) eq '.csv');
      next if (lc($infile) eq 's-model001.csv');

      #
      # スクリプトファイルを読み出してテキストに変換する
      {
        my ($l, $tname, $outfile, $nest, $fhi);
        $tname = substr($infile, 0, length($infile) - 4);
        $outfile = $outdir . $tname . '.txt';
        unless (&wopen($ecd, $outfile)) {
          print STDERR decode('cp932', $outfile) . " に出力できません\n";
          exit 1;
        }
        &wout($bom);
        $nest = 0;
        $fhi = IO::Handle -> new();
        unless (open($fhi, "<:encoding(UTF-16)", $pa)) {
          print STDERR decode('cp932', $pa) . " のオープンに失敗しました\n";
          exit 1;
        }
        while ($l = <$fhi>) {
          #
          # 一行分の処理
          chomp $l;
          next if (substr($l, 0, 2) eq '$$');
          next if (substr($l, 0, 1) eq '#');
          next if (substr($l, 0, 1) eq '[');
          if ($l =~ /^FILEBLOCKHEAD,(.+)\\(.+),$/) {
            my ($e, $h);
            $e = $1;
            $h = $2;
            $h = substr(&conv2jpn($h, 'FILEBLOCKHEAD'), 1);
            $e = &conv2jpn($e, 'FILEBLOCKHEAD');
            &wout($h . ' -' . $e . "\n");
            $nest = 1;
          }
          elsif ($l =~ /^HEADER,.*,.*,(.+),$/) {
            my ($str);
            $str = $1;
            &wout($indent . $str . "\n");
            $nest = 2;
          }
          elsif ($l =~ /^INITJUDGE,(.+),$/) {
            my ($param, $aval, $mes);
            $param = $1;
            if ($param =~ /^(.*?)\\(.*)$/) {
              $aval = $1;
              $param = $2;
              $mes = substr(&conv2jpn($param, 'INITJUDGE ' . $aval), 1);
            }
            else {
              $mes = $unkb . $l . $unke;
            }
            &wout($indent x $nest . $mes . "\n");
            $nest ++;
          }
          elsif ($l =~ /^RETURN,,$/) {
            &wout($indent x $nest . "スクリプト終了\n") if ($nest > 0);
          }
          elsif ($l =~ /^ENDIF,,$/) {
            $nest --;
          }
          elsif ($l =~ /^FOOTER,,$/) {
            $nest = 1;
            &wout("\n");
          }
          elsif ($l =~ /^FILEBLOCKEND,,$/) {
            $nest = 0;
            &wout("\n");
          }
          elsif ($l =~ /^VARIABLE_SET,(.*),(.*),$/) {
            my ($lhand, $rhand);
            $lhand = $1;
            $rhand = $2;
            &wout($indent x $nest . '変数');
            &wout(&conv2jpn($lhand, 'VARIABLE_SET') . ' =');
            &wout(&conv2jpn($rhand, 'VARIABLE_SET') . "\n");
          }
          elsif ($l =~ /^VARIABLE_CALC,(.*?),(.*),$/) {
            my ($lhand, $rhand);
            $lhand = $1;
            $rhand = $2;
            &wout($indent x $nest . '値');
            &wout(&conv2jpn($lhand, 'VARIABLE_CALC') . ' =');
            &wout(&conv2jpn($rhand, 'VARIABLE_CALC') . "\n");
          }
          elsif ($l =~ /^FILEJUMP,.*,(.*),$/) {
            my ($str);
            $str = $1;
            &wout($indent x $nest . 'ジャンプ ' . $str . "\n");
          }
          elsif ($l =~ /^(.*?),(.*),$/) {
            my ($cmd, $param);
            $cmd = $1;
            $param = $2;
            &wout($indent x $nest);
            if ($cmd eq 'IF') {
              &wout('条件分岐');
              $nest ++;
            }
            elsif ($cmd eq 'RECAST') {
              &wout('再発動時間');
              $nest ++;
            }
            elsif ($cmd eq 'TALK') {
              &wout('話す');
            }
            elsif ($cmd eq 'WAIT') {
              &wout('ウェイト');
            }
            elsif ($cmd eq 'FLAG_SET') {
              &wout('フラグ');
            }
            elsif ($cmd eq 'MOVE_SET') {
              &wout('移動');
            }
            elsif ($cmd eq 'MOVE_WAIT') {
              &wout('移動待ち');
            }
            elsif ($cmd eq 'EMOTION_SET') {
              &wout('エモーション');
            }
            elsif ($cmd eq 'EMOTION_WAIT') {
              &wout('エモーション待ち');
            }
            elsif ($cmd eq 'TARGET_SET') {
              &wout('ターゲット');
            }
            elsif ($cmd eq 'VOICE_WAIT') {
              &wout('ボイス待ち');
            }
            else {
              &wout($unkb . $cmd . $unke);
            }
            &wout(&conv2jpn($param, $cmd) . "\n");
          }
          else {
            &wout($unkb . $l . $unke . "\n");
          }
        }
        close $fhi;
        &wclose();
      }
    }
    closedir $dh;
  }
  else {
    print STDERR decode('cp932', $indir) . " にアクセスできません\n";
    exit 1;
  }

#
# スクリプトの一部(\ や , で区切られている)を、日本語に変換する

sub conv2jpn {
  my $instr = $_[0];
  my $context = $_[1];
  my (@kw, $i, $rv, @unit, $suffix);
  @kw = split(/[\\,]/, $instr);
  $rv = '';
  for ($i = 0; $i < @kw; $i ++) {
    my ($rv0);
    $rv .= ' ';
    $rv0 = '';
    if ($kw[$i] eq 'variable') {
      $rv0 = '変数';
    }
    elsif ($kw[$i] eq 'target') {
      $rv0 = 'ターゲット';
    }
    elsif ($kw[$i] eq 'master') {
      $rv0 = '主人';
      if ($context =~ /INITJUDGE/) {
        $suffix = 'を見つけたとき' if ($context =~ /find/);
        $suffix = 'に話しかけられたとき' if ($context =~ /talk/);
      }
    }
    elsif ($kw[$i] eq 'friend') {
      $rv0 = 'フレンド';
      if ($context =~ /INITJUDGE/) {
        $suffix = 'を見つけたとき' if ($context =~ /find/);
        $suffix = 'に話しかけられたとき' if ($context =~ /talk/);
      }
    }
    elsif ($kw[$i] eq 'robo') {
      if ($i > 0 and substr($kw[$i - 1], -1) eq '=') {
        $rv0 = 'キャラドル';
      }
    }
    elsif ($kw[$i] eq 'npc') {
      $rv0 = 'NPC';
      $suffix = 'を見つけたとき' if ($context =~ /INITJUDGE/);
    }
    elsif ($kw[$i] eq 'date') {
      if ($context =~ /TARGET_SET/) {
        $rv0 = '月日指定';
        @unit = qw(月 日);
      }
      elsif ($context =~ /INITJUDGE/ || $i >= 2) {
        $rv0 = '日時';
      }
      else {
        $rv0 = '時間パラメータ';
      }
    }
    elsif ($kw[$i] eq 'distance') {
      $rv0 = '距離';
    }
    elsif ($kw[$i] eq 'flag') {
      $rv0 = 'フラグ';
    }
    elsif ($kw[$i] eq 'basejudge') {
      $rv0 = '基本条件';
    }
    elsif ($kw[$i] eq 'bloodgroup') {
      $rv0 = '血液型';
    }
    elsif ($kw[$i] eq 'birthday') {
      $rv0 = '誕生日';
    }
    elsif ($kw[$i] eq '==') {
      $rv0 = 'が次の値に等しい';
    }
    elsif ($kw[$i] eq '!=') {
      $rv0 = 'が次の値に等しくない';
    }
    elsif ($kw[$i] eq 'anybody') {
      $rv0 = '誰でも';
      if ($context =~ /INITJUDGE/) {
        $suffix = 'を見つけたとき' if ($context =~ /find/);
        $suffix = 'に話しかけられたとき' if ($context =~ /talk/);
      }
    }
    elsif ($kw[$i] eq 'name') {
      if ($context =~ /EMOTION_SET/) {
        $rv0 = 'エモーション名';
      }
      else {
        $rv0 = '名前指定';
        if ($context =~ /INITJUDGE/) {
          $suffix = 'を見つけたとき' if ($context =~ /find/);
          $suffix = 'に話しかけられたとき' if ($context =~ /talk/);
        }
      }
    }
    elsif ($kw[$i] eq 'time_clock') {
      $rv0 = '現在時刻' if ($i > 0 and $kw[$i - 1] eq 'date');
    }
    elsif ($kw[$i] eq 'value_dow') {
      $rv0 = '曜日' if ($i > 0 and substr($kw[$i - 1], -1) eq '=');
    }
    elsif ($kw[$i] eq 'value_date') {
      if ($i > 0 and substr($kw[$i - 1], -1) eq '=') {
        $rv0 = '日時';
        @unit = qw(月 日);
      }
    }
    elsif ($kw[$i] eq 'value_dow_clock') {
      $rv0 = '曜日と時刻';
      @unit = ('', '時', '分');
      $suffix = 'になったら' if ($context =~ /INITJUDGE/);
    }
    elsif ($kw[$i] eq 'value_date_clock') {
      if ($context =~ /INITJUDGE/) {
        $rv0 = '日時と時刻';
        $suffix = 'になったら';
      }
      elsif ($i > 1 && $kw[$i - 1] eq 'date') {
        $rv0 = '日時と時刻';
      }
      else {
        $rv0 = '月日と時刻';
      }
      @unit = qw(月 日 時 分);
    }
    elsif ($kw[$i] eq 'count_time_move') {
      $rv0 = '連続移動時間' if ($i > 0 and $kw[$i - 1] eq 'date');
    }
    elsif ($kw[$i] eq 'cont_time_move') {
      $rv0 = '連続移動時間' if ($i > 0 and $kw[$i - 1] eq 'date');
    }
    elsif ($kw[$i] eq 'value_time') {
      $rv0 = '時間';
      @unit = qw(時 分 秒);
    }
    elsif ($kw[$i] eq 'time_login') {
      $rv0 = '今ログイン経過時間' if ($i > 0 and $kw[$i - 1] eq 'date');
    }
    elsif ($kw[$i] eq 'trip_time') {
      $rv0 = '外出してからの移動時間' if ($i > 0 and $kw[$i - 1] eq 'date');
    }
    elsif ($kw[$i] eq 'count_distance_move') {
      $rv0 = '連続移動距離' if ($i > 0 and $kw[$i - 1] eq 'distance');
    }
    elsif ($kw[$i] eq 'cont_distance_move') {
      $rv0 = '連続移動距離' if ($i > 0 and $kw[$i - 1] eq 'distance');
    }
    elsif ($kw[$i] eq 'login_distance_move') {
      $rv0 = '今ログイン移動距離' if ($i > 0 and $kw[$i - 1] eq 'distance');
    }
    elsif ($kw[$i] eq 'trip_distance_move') {
      if ($i > 0 and $kw[$i - 1] eq 'distance') {
        $rv0 = '外出してからの移動距離';
      }
    }
    elsif ($kw[$i] eq '>=') {
      $rv0 = 'が次の値以上';
    }
    elsif ($kw[$i] eq '<=') {
      $rv0 = 'が次の値以下';
    }
    elsif ($kw[$i] eq '>') {
      $rv0 = 'が次の値より大きい';
    }
    elsif ($kw[$i] eq '<') {
      $rv0 = 'が次の値より小さい';
    }
    elsif ($kw[$i] eq 'value_distance') {
      $rv0 = '距離設定';
      @unit = qw(km m);
    }
    elsif ($kw[$i] eq 'doll') {
      $rv0 = 'キャラドル';
    }
    elsif ($kw[$i] eq 'global') {
      $rv0 = 'グローバル';
    }
    elsif ($kw[$i] eq 'on_off') {
      $rv0 = 'ON/OFF';
    }
    elsif ($kw[$i] eq 'on') {
      $rv0 = 'ON' if ($i > 0 and $kw[$i - 1] eq 'on_off');
    }
    elsif ($kw[$i] eq 'off') {
      $rv0 = 'OFF' if ($i > 0 and $kw[$i - 1] eq 'on_off');
    }
    elsif ($kw[$i] eq 'value_number') {
      $rv0 = '数値';
    }
    elsif ($kw[$i] eq 'find') {
      $rv0 = '見つけた';
    }
    elsif ($kw[$i] eq 'other') {
      $rv0 = 'その他';
    }
    elsif ($kw[$i] eq 'wait_time') {
      $rv0 = '待機時間';
      @unit = qw(分 秒);
      $suffix = '操作しなかったら' if ($context =~ /INITJUDGE/);
    }
    elsif ($kw[$i] eq 'move_time') {
      $rv0 = '移動時間';
      @unit = qw(分 秒);
      $suffix = '移動し続けたら' if ($context =~ /INITJUDGE/);
    }
    elsif ($kw[$i] eq 'move_distance') {
      $rv0 = '移動距離';
      @unit = qw(km m);
      $suffix = '以上移動したら' if ($context =~ /INITJUDGE/);
    }
    elsif ($kw[$i] eq 'value_clock') {
      $rv0 = '時刻';
      @unit = qw(時 分);
      $suffix = 'になったら' if ($context =~ /INITJUDGE/);
    }
    elsif ($kw[$i] eq 'nocheck') {
      $rv0 = '必ず実行';
      $rv0 .= ' 再実行時間になったら' if ($context =~ /INITJUDGE/);
    }
    elsif ($kw[$i] eq 'value_world') {
      $rv0 = '場所';
      $suffix = 'に移動したら' if ($context =~ /INITJUDGE/);
    }
    elsif ($kw[$i] eq 'listen') {
      $rv0 = '聞いた';
      $suffix = 'を含む言葉を聞いたら';
    }
    elsif ($kw[$i] eq 'emotion') {
      $rv0 = 'エモーション';
      $suffix = 'をしていたら' if ($i == 0);
    }
    elsif ($kw[$i] eq 'value_is_login') {
      $rv0 = 'ログイン';
    }
    elsif ($kw[$i] eq 'value_is_areachange') {
      $rv0 = 'エリアチェンジ';
    }
    elsif ($kw[$i] eq 'value_is_myroom') {
      $rv0 = 'マイルーム';
    }
    elsif ($kw[$i] eq 'value_random') {
      $rv0 = '乱数';
    }
    elsif ($kw[$i] eq 'indoor') {
      $rv0 = '室内';
    }
    elsif ($kw[$i] eq 'outdoor') {
      $rv0 = '室外';
    }
    elsif ($kw[$i] eq 'talk') {
      $rv0 = '話しかけられた';
    }
    elsif ($kw[$i] eq 'linkonly') {
      $rv0 = '設定なし 自動では動作しません。';
      $rv0 .= 'ここをジャンプ指定した場合、動作します。';
    }
    elsif ($kw[$i] eq 'msec') {
      $rv0 = 'ミリ秒';
    }
    elsif ($kw[$i] eq 'sec') {
      $rv0 = '秒';
    }
    elsif ($kw[$i] eq 'min') {
      $rv0 = '分';
    }
    elsif ($kw[$i] eq 'nob') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = 'うなずく';
      }
    }
    elsif ($kw[$i] eq 'no') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '首を横に振る';
      }
    }
    elsif ($kw[$i] eq 'tumble') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '転ぶ';
      }
    }
    elsif ($kw[$i] eq 'joy') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = 'ガッツポーズ';
      }
    }
    elsif ($kw[$i] eq 'signifier') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '指をさす';
      }
    }
    elsif ($kw[$i] eq 'bye') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '手を振る';
      }
    }
    elsif ($kw[$i] eq 'speak') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '話す';
      }
    }
    elsif ($kw[$i] eq 'sheepish') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '恥ずかしがる';
      }
    }
    elsif ($kw[$i] eq 'bashful') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '照れる';
      }
    }
    elsif ($kw[$i] eq 'disappoint') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = 'がっかりする';
      }
    }
    elsif ($kw[$i] eq 'fluster') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '慌てる';
      }
    }
    elsif ($kw[$i] eq 'survey') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = 'キョロキョロする';
      }
    }
    elsif ($kw[$i] eq 'loath') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '頭を抱える';
      }
    }
    elsif ($kw[$i] eq 'happy') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '嬉しい';
      }
    }
    elsif ($kw[$i] eq 'sad') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '哀しい';
      }
    }
    elsif ($kw[$i] eq 'cry') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '泣く';
      }
    }
    elsif ($kw[$i] eq 'cadge') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = 'ねだる';
      }
    }
    elsif ($kw[$i] eq 'urprised') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '驚く';
      }
    }
    elsif ($kw[$i] eq 'surprised') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '驚く';
      }
    }
    elsif ($kw[$i] eq 'clap') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '拍手';
      }
    }
    elsif ($kw[$i] eq 'peace') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = 'ピース';
      }
    }
    elsif ($kw[$i] eq 'dither') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '迷う';
      }
    }
    elsif ($kw[$i] eq 'openarms') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '両手を広げる';
      }
    }
    elsif ($kw[$i] eq 'think') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '考え事';
      }
    }
    elsif ($kw[$i] eq 'swagger') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '威張る';
      }
    }
    elsif ($kw[$i] eq 'secret') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '内緒';
      }
    }
    elsif ($kw[$i] eq 'promise') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '約束';
      }
    }
    elsif ($kw[$i] eq 'sit') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = '座る';
      }
    }
    elsif ($kw[$i] eq 'wait1') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = 'ウェイトモーション1';
      }
    }
    elsif ($kw[$i] eq 'wait2') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = 'ウェイトモーション2';
      }
    }
    elsif ($kw[$i] eq 'wait3') {
      if ($i > 0 && ($context =~ /EMOTION_SET/) || $kw[$i - 1] ne 'name') {
        $rv0 = 'ウェイトモーション3';
      }
    }
    elsif ($kw[$i] eq 'walk') {
      $rv0 = '歩く';
    }
    elsif ($kw[$i] eq 'ran') {
      $rv0 = '走る';
    }
    elsif ($kw[$i] eq 'run') {
      $rv0 = '走る';
    }
    elsif ($kw[$i] eq 'near_master') {
      $rv0 = '主人に近づく';
    }
    elsif ($kw[$i] eq 'away_master') {
      $rv0 = '主人からすこし離れる';
    }
    elsif ($kw[$i] eq 'near_target') {
      $rv0 = 'ターゲットに近づく';
    }
    elsif ($kw[$i] eq 'away_target') {
      $rv0 = 'ターゲットから離れる';
    }
    elsif ($kw[$i] eq 'move_default_outdoor') {
      $rv0 = '主人についていく';
    }
    elsif ($kw[$i] eq 'move_default_indoor') {
      $rv0 = '部屋をうろうろ';
    }
    elsif ($kw[$i] eq 'near') {
      $rv0 = 'が一番近い';
      @unit = ();
    }
    elsif ($kw[$i] eq 'far') {
      $rv0 = 'が一番遠い';
      @unit = ();
    }
    elsif ($kw[$i] eq 'a') {
      if ($i > 0 and $kw[$i - 1] eq 'bloodgroup') {
        $rv0 = 'A';
        @unit = ();
      }
    }
    elsif ($kw[$i] eq 'b') {
      if ($i > 0 and $kw[$i - 1] eq 'bloodgroup') {
        $rv0 = 'B';
        @unit = ();
      }
    }
    elsif ($kw[$i] eq 'o') {
      if ($i > 0 and $kw[$i - 1] eq 'bloodgroup') {
        $rv0 = 'O';
        @unit = ();
      }
    }
    elsif ($kw[$i] eq 'ab') {
      if ($i > 0 and $kw[$i - 1] eq 'bloodgroup') {
        $rv0 = 'AB';
        @unit = ();
      }
    }
    elsif ($kw[$i] eq '~') {
      if ($i > 1 and $kw[$i - 2] eq 'value_random') {
        $rv0 = '～';
        @unit = ();
      }
    }

    if ($rv0 eq '') {
      my ($p);
      $p = $kw[$i];
      $p = $1 if ($p =~ /^"(.*)"$/);
      $p = $strb . $p . $stre if ($p =~ /\s/);
      $rv .= $p . $unit[0];
      shift @unit;
    }
    else {
      $rv .= $rv0;
    }

  }
  $rv .= ' ' . $suffix unless ($suffix eq '');
  return $rv;
}

{ # ファイル出力関数群

  my ($fho, $buf);   # static変数

  sub wopen {
    my $ecd = $_[0];    # エンコーディング
    my $fn = $_[1];     # ファイル名
    $fho = IO::Handle -> new();
    if (open($fho, '>:' . $ecd, $fn)) {
      $buf = '';
      return 1;
    }
    else {
      return 0;
    }
  }

  sub wout {
    my $str = $_[0];    # 出力文字列
    $buf .= $str;
    if (length($buf) >= 255) {
      $SIG{'__WARN__'} = \&ignoresig;
      print $fho $buf;
      undef $SIG{'__WARN__'};
      $buf = '';
    }
  }

  sub wclose {
    $SIG{'__WARN__'} = \&ignoresig;
    print $fho $buf;
    close $fho;
    undef $SIG{'__WARN__'};
  }

  sub ignoresig {
  }
}

__END__
